use std::net::SocketAddr;

use anyhow::Context;
use mad::Component;

use crate::state::SharedState;

include!("gen/nodata.rs");

#[derive(Clone)]
pub struct GrpcServer {
    state: SharedState,
    host: SocketAddr,
}

impl GrpcServer {
    pub fn new(state: &SharedState, host: SocketAddr) -> Self {
        Self {
            state: state.clone(),
            host,
        }
    }
}

#[tonic::async_trait]
impl no_data_server::NoData for GrpcServer {
    async fn publish_event(
        &self,
        request: tonic::Request<PublishEventRequest>,
    ) -> std::result::Result<tonic::Response<PublishEventResponse>, tonic::Status> {
        let req = request.into_inner();

        tracing::debug!(
            topic = req.topic,
            key = req.key,
            value = std::str::from_utf8(&req.value).ok(),
            "handling event"
        );

        Ok(tonic::Response::new(PublishEventResponse {}))
    }
}

#[axum::async_trait]
impl Component for GrpcServer {
    fn name(&self) -> Option<String> {
        Some("grpc_server".into())
    }

    async fn run(
        &self,
        _cancellation_token: tokio_util::sync::CancellationToken,
    ) -> Result<(), mad::MadError> {
        tracing::info!("grpc listening on: {}", self.host);

        tonic::transport::Server::builder()
            .add_service(no_data_server::NoDataServer::new(self.clone()))
            .serve(self.host)
            .await
            .context("grpc server failed")
            .map_err(mad::MadError::Inner)?;

        Ok(())
    }
}
